/*****************************************************************************
HOW CBO ADJUSTS FOR SURVEY UNDERREPORTING OF TRANSFER INCOME IN ITS
DISTRIBUTIONAL ANALYSES

SUPPLEMENTAL CODE

BILAL HABIB
CONGRESSIONAL BUDGET OFFICE
JULY 2018
*****************************************************************************

Please read the readme.txt file before using this code. 

This code was written and tested using Stata 14.1.

This code is a supplement to CBO's working paper 2018-07: "How CBO Adjusts for 
Survey Underreporting of Transfer Income in its Distributional Analyses." It 
should be used with the data sets released along with that paper. Along with 
those files, this code allows researchers to replicate CBO's imputations for 
means-tested transfers in the Current Population Survey (CPS) for each year 
of the analysis (CPS 1980 to CPS 2017) for three of the largest means-tested 
transfer programs: Medicaid and the Children's Health Insurance Program (CHIP); 
Supplemental Nutrition Assistance Program (SNAP); and Supplemental Security 
Income (SSI). 

Researchers interested in using CBO's imputations for their own research 
should note that those imputations were designed specifically for CBO's
quintile-level national analyses of income distribution and should not be
used for state-level analyses, causal analyses, or policy simulations. */

clear all
set more off
capture log close


*****************************************************************************
*** USER INPUT
*****************************************************************************

// Enter the years for which imputation output is required (available years
// are 1980 to 2017).
local years "[enter years separated by a space]"

// Enter the folder in which the data from the CBO website are saved.
local input_path  "[enter path of input folder]"

// Enter the folder into which the output and log should be saved. Make sure 
// that this folder exists. 
local output_path "[enter path of output folder]"


*****************************************************************************
*** PREPARE DATA TO RUN THE ALGORITHM BY SUBGROUP
*****************************************************************************

log using "`output_path'\cbo_distribution_meanstest_imputations_log.txt", replace text

// Read in the targets file and save in Stata format to merge later.
insheet using "`input_path'\cbo_distribution_meanstest_total_recipients.csv", clear
save "`input_path'\cbo_distribution_meanstest_total_recipients.dta", replace

// The imputation method is conducted separately for eight subgroups across the 
// three means-tested transfer programs. For more details, see section 3 of the 
// working paper.
#delimit ;
local subgroups "mcaid_adult mcaid_child mcaid_senior mcaid_disabled 
				snap_allhhd
				ssi_adult ssi_child ssi_senior";
#delimit cr

// Loop through each year.
foreach yr in `years' {

	insheet using "`input_path'\cbo_distribution_meanstest_probabilities_`yr'.csv", clear

	// Merge in the targets data.
	qui merge m:1 year using "`input_path'\cbo_distribution_meanstest_total_recipients.dta"
	drop if _merge != 3
	drop _merge

	// Loop through the three means-tested programs to create the variables 
	// for the imputed recipiency status and imputed benefit values. The algorithm 
	// will fill in these variables as it moves through the subgroups.
	foreach prog in mcaid snap ssi {

		gen `prog'_impute = 0
		label var `prog'_impute "Post-imputation recipient of `prog'"

		gen `prog'_impute_val = 0
		label var `prog'_impute_val "Post-imputation value of `prog' benefits"

	}


	*****************************************************************************
	*** LOOP THROUGH SUBGROUPS
	*****************************************************************************
	
	foreach grp in `subgroups' {

		// Set the relevant target as a scalar.		
		qui sum `grp'_target_recipients
		local `grp'_target = r(mean)

		// Subgroups are named according to the scheme "program_subgroup", 
		// e.g., "ssi_adult". But some variables are at the program level. 
		// This code creates a "program" local for each subgroup.
		local prog = substr("`grp'", 1, strpos("`grp'", "_") - 1)

		// Calculate the total numbers of reported and reported + imputed recipients.
		qui sum `prog'_report [w=hsup_wgt] if `prog'_report == 1 & `grp' == 1
		local tot_`grp'_report = r(sum_w)
		local tot_`grp'_impute = r(sum_w)

		// Calculate the total number of units (recipient or nonrecipient) in a subgroup.
		qui sum `grp' [w=hsup_wgt] if `grp' == 1
		local total_`grp' = r(sum_w)

		// Create an adjustment factor for the random number. There is no 
		// adjustment in the first round, so the factor is set to 1.
		local i = 0
		local adj_factor = 1

		// The algorithm is designed to get the final reported + imputed total to 
		// within one weighted household of the target. This algorithm takes the 
		// maximum value of hsup_wgt as a proxy for one weighted household.
		qui sum hsup_wgt, detail
		local onehhd = r(max)
		
		// Loop until the final imputed + reported total is within one weighted 
		// household of the target.
		while abs(`tot_`grp'_impute' - ``grp'_target') > `onehhd' {

			// Add to the round count.
			local ++i

			// To start the round, set the imputed + reported series to be equal 
			// to the reported series.
			gen `grp'_impute_`i' = `prog'_report if `grp' == 1

			// If the number of reporting units is below the target (typically 
			// the case), assign receipt if a unit's estimated probability of 
			// receipt exceeds the random number.
			if `tot_`grp'_report' < ``grp'_target' ///
				replace `grp'_impute_`i' = 1 ///
				if `prog'_prob > (`prog'_rand * `adj_factor') & `grp' == 1

			// In the few cases where the number of reporters exceeds the target, 
			// remove reported receipt if a unit's probability is lower than the 
			// random number.
			else ///
				replace `grp'_impute_`i' = 0 ///
				if `prog'_prob < (`prog'_rand * `adj_factor') & `grp' == 1

			// Calculate the new total.
			qui sum `grp'_impute_`i' [w=hsup_wgt] if `grp'_impute_`i' == 1
			local tot_`grp'_impute = r(sum_w)

			// Calculate the adjustment factor (depending on whether the algorithm
			// is correcting for underreporting or overreporting).
			if `tot_`grp'_report' < ``grp'_target' ///
				local adj_factor = `adj_factor' * (`tot_`grp'_impute' - ///
					`tot_`grp'_report') / (``grp'_target' - `tot_`grp'_report')
			else ///
				local adj_factor = `adj_factor' / ((`tot_`grp'_impute' - ///
					`tot_`grp'_report') / (``grp'_target' - `tot_`grp'_report'))
			
		} // End the while loop after the target for the subgroup has been reached.


		*****************************************************************************
		*** POST-IMPUTATION TRANSFER RECEIPT AND INCOME ASSIGNMENT
		*****************************************************************************

		// If no imputation was necessary, set the imputed + reported series equal
		// to the reported series.
		if `i' == 0 ///
			gen `grp'_impute_0 = `prog'_report

		// Set the imputed recipiency status for the program equal to the final
		// round of imputations for that particular subgroup.
		replace `prog'_impute = `grp'_impute_`i' if `grp' == 1

		// For each imputed unit, assign the potential value from the dataset. 
		replace `prog'_impute_val = `prog'_potential_val ///
			if `prog'_impute == 1 & `grp' == 1

		// Drop the intermediate imputation variables.
		capture drop `grp'_impute_*

	} // End the subgroup loop after transfer income assignment is complete.

	// After all the imputations have been completed, drop the "potential value" 
	// variable, and the target variables.
	drop *potential* *target*

	save "`output_path'\cbo_distribution_meanstest_imputations_`yr'.dta", replace

}

log close
